/* **************************************************************** */
/* **************************************************************** */
/* *                                                              * */ 
/* *    Header file to Anonymizer library                         * */
/* *    Version 5.x              2016-10-10                       * */ 
/* *   --------------------------------------------------------   * */
/* *                                                              * */
/* *    Eyedea Recognition s.r.o. (C) 2016                        * */
/* *                                                              * */
/* *    Contact:                                                  * */   
/* *                web: http://www.eyedea.cz                     * */ 
/* *              email: info@eyedea.cz                           * */
/* *                                                              * */
/* **************************************************************** */
/* **************************************************************** */

#ifndef _EYE_ANONIMIZE_H
#define _EYE_ANONIMIZE_H

/*! \addtogroup AnonymizerTypes
 * @{
 */

/*! \brief Handle to Anonymizer SDK library
    \see anInit
*/
typedef void* ANState;

/* Anonymization modules flags used in AnParams.iMode */
/*! \brief Flag for face anonymization. 
 * 
 *  When satisfied AnParams.iMode & ANONYMIZE_FACE the face anonymization is performed. \see AnParams.iMode */
#define ANONYMIZE_FACE 1

/*! \brief Flag for licence plate anonymization.
 * 
 * When satisfied AnParams.iMode & ANONYMIZE_LP the licence plate anonymization is performed. \see AnParams.iMode */
#define ANONYMIZE_LP   2

/*! \brief Region Of Interest
*  Region Of Interest (ROI) defines image area where anonymization is applied. The ROI is defined by its top left point and size.
*  The active pixels of regions are defined by closed ranges <x,x+width-1>, <y,y+height-1>
*/
typedef struct
{
    int x;                 /*!< define top left column of ROI */
    int y;                 /*!< define top left row of ROI */
    unsigned int width;    /*!< width of ROI */
    unsigned int height;   /*!< height of ROI */
} AnROI;

/*! \brief Image buffer enclosing structure
*/
typedef struct
{
    unsigned char *raw_data; /*!< byte array of data (raw image data, jpeg encoded data) */
    size_t         length;   /*!< length of buffer */
}AnBuffer;


/*! \brief Control parameters 
 *
 * Set of parameters controling anonymization process. AnParams enables to define parameters in runtime like ROI definition, anonymized object type, output image quality etc.
 * 
 * Anonymization contains two detection modules (Face and Licence Plate), for both the control of balance between true positive detections and false alarms can be balanced by dFaceConfidenceThr, 
 * dLpConfidenceThr parameters. Higher value -> less false detections & lower detection rate, Lower value -> higher detection rate, more false detections.
 * 
 * Anonymization area and anonymization degree is controled by dFaceDetSizeFactor & iFaceNumBlurPass (dLpDetSizeFactor & iLpNumBlurPass respectively)
 * 
 * For visualisation purposes the iShowDetection flag is used. Detections are then highlighted insted of blurred
 * 
 * See Developer Guide for more details.
 */
typedef struct
{
    AnROI  ROI;                /*!< Region of interest - the area scanned for detections default from_col=0, to_col=MAX_INT, from_row=0, to_row=MAX_INT */
    int    mode;               /*!< Flag indicating whether to anonymize faces or license plates or both. default ANONYMIZE_FACE | ANONYMIZE_LP */
    int    showDetections;     /*!< Debugging flag indicating, that detections will be highlighted instead of blurred */
    int    jpegQuality;        /*!< Output image jpeg quality 0..100, default 90 */
    int    keepEXIF;           /*!< EXIF information in JPG files will be preserved. NOTE that EXIF thumbnail is not anonymized and that nonstandard EXIFs may not be preserved. */
    int    panoramic;          /*!< Enable copying of left and right borders to detect objects on the edges of panoramatic photos */
    
    double faceConfidenceThr;  /*!< The minimal confidence of a face to be anonymized. (Detector return confidences > 0) */
    double faceDetSizeFactor;  /*!< Parameter to enlarge anonymized area ( < 1.0 - smaller than detection, ==1.0 - exactly as detection; >1.0 - larger than detection) */
    int    faceNumBlurPass;    /*!< Number of blur passes over face detection */
    
    double lpConfidenceThr;    /*!< The minimal confidence of a license plate to be anonymized. (Detector return confidences > 0)*/
    double lpDetSizeFactor;    /*!< Parameter to enlarge anonymized area ( < 1.0 - smaller than detection, ==1.0 - exactly as detection; >1.0 - larger than detection) */
    int    lpNumBlurPass;      /*!< number of blur passes over lp detection */
    
    int    verbose;            /*!< Enable logging anonymization progress on to stdout */
    
} AnParams; /*!< Control parameters */


/*! @} */
    

/*! \addtogroup AnonymizerAPI
 * @{
 */


/*! \brief  Initializes/loads Anonymizer engine.
 * 
 * anInit is the initialization function which loads and set-ups all detections models. Input parameters of the function are
 * path to "modules" directory of Anonymizer SDK which contains configuration files and directory detection models and config filename.
 * Function returns zero on success or error code otherwise.
 * \param[in]  sdk_directory Path of the AnonymizeSDK.
 * \param[in]  ini_filename Config file name (if NULL, default "config.ini" is used)
 * \param[out] state pointer to ANState module handle.
 * \return Zero on success, error code on failure.
 */
ER_FUNCTION_PREFIX  int anInit(const char* sdk_directory, const char* ini_filename, ANState *state);


/*! \brief  anInit function pointer
 * \see anInit
 */
typedef int (*fcn_anInit)(const char*, const char*, ANState *);

/*! \brief  Frees engine state structure.
    \param[in]  state Anonymizer state, for state initalization see \ref anInit function.
*/
ER_FUNCTION_PREFIX  void anFree(ANState state);

/*! \brief  anFree function pointer
 * \see anFree
 */
typedef void (*fcn_anFree)(ANState);

/*! \brief  Load image from image file and process the anonymization. Result save as a jpeg image.
 * 
  Main function for anonymization. Function loads image from a file, run anonymization engine and save the anonymized image
  as jpeg file. Function return 0 on success or error code.
  \param[in]  src_image_filename Input image filename.
  \param[in]  dst_image_filename Output image filename (jpeg). 
  \param[in]  params  Pointer to anonymization parameters, if NULL default values are used.
  \param[in]  state   Anonymizer state, for state initalization see \ref anInit function.
  \return Zero on success, error code on failure.
  \see example-files.cpp
*/
ER_FUNCTION_PREFIX  int   anAnonymize(const char*  src_image_filename, 
                                   const char*  dst_image_filename, 
                                   AnParams*    params,
                                   ANState      state);

/*! \brief  anAnonymize function pointer
 * \see anAnonymize
 */
typedef int(*fcn_anAnonymize)(const char*, const char*, AnParams*, ANState);

/*! Main function for anonymization. Function takes as an input parameter raw image data aligned row by row and run anonymization engine. The anonymized image is returned in 
    ouput parmeter in format of input data. Output data buffer (dstBuffer) has to be freed! Call anFreeBuffer(dstBuffer) when you finish job.
    \brief  Processes image data buffer, i.e. detects and blurs faces and license plates.
    \param[in]  srcBuffer Input structure with BGR data buffer - row-wise, 3 bytes (uchar) per pixel; index = 3*col + row*3*width
    \param[in]  width   Image width
    \param[in]  height  Image height
    \param[in]  params  Pointer to anonymization parameters, if NULL default values are used.
    \param[in]  state   Anonymizer state, for state initalization see \ref anInit function.
    \param[out] dstBuffer Anonymized data buffer (BGR). Call anFreeBuffer() to free buffer.
    \return Zero on success, error code on failure.
	\see example-buffers.cpp
*/
ER_FUNCTION_PREFIX  int   anAnonymizeImageBuffer(AnBuffer      srcBuffer, 
                                         unsigned int     width,
                                         unsigned int     height,
                                         AnParams*        params,
                                         ANState          state,
                                         AnBuffer*        dstBuffer );

/*! \brief  anAnonymizeImageBuffer function pointer
 * \see anAnonymizeImageBuffer
 */
typedef int(*fcn_anAnonymizeImageBuffer)(AnBuffer, int, int, AnParams*, ANState, AnBuffer*);

/*! Frees memory previously allocated by anAnonymizeImageBuffer or anAnonymizeJpegBuffer functions.
    \brief  Frees the image buffer filled by Anonymizer SDK functions.
    \param  buffer AnBuffer structure.
*/
ER_FUNCTION_PREFIX  void anFreeBuffer(AnBuffer buffer);

/*! \brief  anFreeBuffer function pointer
 * \see anFreeBuffer
 */
typedef void (*fcn_anFreeBuffer)(AnBuffer buffer);

/*! Main function for anonymization. Function takes as an input parameter JPEG image buffer (compressed data) which anonymize. The anonymized image is returned in 
    ouput parameter also as JPEG image buffer. Function return 0 on success or error code.
    \brief  Processes the JPEG image buffer, i.e. detects and blurs faces and license plates.
    \param[in]   srcBuffer  AnBuffer structure with input JPEG buffer.
    \param[in]   params    Pointer to anonymization parameters, if NULL default values are used.
    \param[in]   state     Anonymizer state, for state initalization see \ref anInit function.
    \param[out]  dstBuffer Pointer to AnBuffer structure full filled with anonymized JPEG buffer. Call anFreeBuffer() to free the buffer data.
    \return Zero on success, error code on failure.
	\see example-buffers.cpp
*/
ER_FUNCTION_PREFIX  int  anAnonymizeJpegBuffer(AnBuffer   srcBuffer,
                                               AnParams*  params,
                                               ANState    state,
                                              AnBuffer*   dstBuffer);

/*! \brief  anAnonymizeJpegBuffer function pointer
 * \see anAnonymizeJpegBuffer
 */
typedef int(*fcn_anAnonymizeJpegBuffer)(AnBuffer, AnParams*, ANState, AnBuffer*);

/*! Fill AnParams structure with default values. Pass pointer to static or dynamically allocated AnParams structure
 * to be filled with default values.
 *    \brief  Fill AnParams structure with default values.
 *    \param[out]  parameters Pointer to AnParams structure.
 *    \return Zero on success, error code on failure.
 */
ER_FUNCTION_PREFIX  int anGetDefaultParams(AnParams *parameters);

/*! \brief  anGetDefaultParams function pointer
 * \see anGetDefaultParams
 * \return Zero on success, error code on failure.
 */
typedef void (*fcn_anGetDefaultParams)(AnParams*);

/*! \brief Get Anonymizer version string
*   \param verbose Verbosity flag. */
ER_FUNCTION_PREFIX const char* anVersion(int verbose);

/*! \brief  anVersion function pointer
* \see anVersion
*/
typedef const char* (*fcn_anVersion)(int);

/*! \brief Return error message for suplied error code
\note anGetErrorMsg function is not thread safe. Concurrent call can lead to missinterpreted error codes.
*/
ER_FUNCTION_PREFIX const char* anGetErrorMsg(int ern);

/*! \brief  anGetErrorMsg function pointer
* \see anGetErrorMsg
*/
typedef const char*(*fcn_anGetErrorMsg)(int);

/*! 
@}
*/
#endif

